<?php

/**
 * @file
 * Contains functions only needed for drush integration.
 */

/**
 * Implementation of hook_drush_command().
 */
function neato_drush_command() {
  $items = [];

  $items['neato-generate'] = [
    'description' => 'Create a Neato sub-theme',
    'aliases'     => ['ngt'],
    'arguments'   => [
      'name' => 'Your sub-theme name.',
    ],
    'options'     => [
      'machine_name' => 'A machine-readable name for your theme, optional. Machine name must be lowercase only, with underscores instead of spaces.',
      'description'  => 'Your sub-theme description.',
    ],
    'examples'    => [
      'drush ngt "custom theme name"'                                        => 'Create a sub-theme with the default options.',
      'drush ngt "foo bar" "foo_bar" --description="My custom Neato theme."' => 'Create a sub-theme with additional options.',
    ],
  ];

  return $items;
}

/**
 * Create a Neato sub-theme.
 */
function drush_neato_generate($name) {
  $description = drush_get_option('description');
  $machine_name = drush_get_option('machine_name');

  if (empty($name)) {
    drush_set_error(dt("Please provide a name for the sub-theme.\nUSAGE:\tdrush ngt [name] --machine_name=[machine_name !OPTIONAL] --description=[description !OPTIONAL]\n"));
    return;
  }

  // Filter everything but letters, numbers, underscores, and hyphens.
  $machine_name = !empty($machine_name) ? preg_replace('/[^a-z0-9_-]+/', '', strtolower($machine_name)) : preg_replace('/[^a-z0-9_-]+/', '', strtolower($name));
  // Eliminate hyphens.
  $machine_name = str_replace('-', '_', $machine_name);

  // Find theme paths.
  $neato_path    = drush_get_context('DRUSH_DRUPAL_ROOT') . '/' . drupal_get_path('theme', 'neato');
  $subtheme_path = dirname($neato_path) . '/' . $machine_name;
  if (strpos($subtheme_path, 'sites/all/themes/contrib') !== FALSE) {
    $subtheme_path = str_replace('sites/all/themes/contrib', 'sites/all/themes/custom', $subtheme_path);
    $themes_custom_path = drush_get_context('DRUSH_DRUPAL_ROOT') . '/sites/all/themes/custom';
    if (!file_exists($themes_custom_path)) {
      mkdir($themes_custom_path, 0755, true);
    }
  }
  $themes_custom_path = isset($themes_custom_path) ? TRUE : FALSE;

  // Make sure bower components are present.
  if (!file_exists("$neato_path/bower_components")) {
    drush_set_error(dt("Please run bower install before generating the subtheme.\n"));
    return;
  }

  // Make a fresh copy of the subtheme.
  $s = drush_copy_dir("$neato_path/STARTER/", $subtheme_path);
  if (empty($s)) {
    return;
  }

  // Copy Bitters to the subtheme folder.
  $b = drush_copy_dir("$neato_path/bower_components/bitters/app/assets/stylesheets/", "$subtheme_path/scss/base", FILE_EXISTS_MERGE);
  if (empty($b)) {
    return;
  }

  // Fix the bitters grid settings neat import (unneeded import).
  drush_op('neato_fix_bitters_neat_import', "$subtheme_path/scss/base/_grid-settings.scss", $themes_custom_path);

  // Rename files and fill in the theme machine name.
  drush_op('rename', "$subtheme_path/STARTER.info.txt", "$subtheme_path/$machine_name.info");
  drush_op('rename', "$subtheme_path/css/STARTER.css", "$subtheme_path/css/$machine_name.css");
  drush_op('rename', "$subtheme_path/scss/STARTER.scss", "$subtheme_path/scss/$machine_name.scss");
  drush_op('rename', "$subtheme_path/templates/base/page.tpl.php.txt", "$subtheme_path/templates/base/page.tpl.php");

  // Update the .info file.
  drush_op('neato_file_str_replace', "$subtheme_path/$machine_name.info", 'Neato Starter', "$name");

  // Change the description of the theme.
  if (!empty($description)) {
    drush_op('neato_file_str_replace', "$subtheme_path/$machine_name.info", 'Custom sub-theme based on the the Neato base theme.', "$description");
  }

  // Create the template.php file with replaced theme name.
  drush_op('neato_file_str_replace', "$subtheme_path/template.php", 'STARTER', "$machine_name");
  // Rename the .info reference to the custom css file.
  drush_op('neato_file_str_replace', "$subtheme_path/$machine_name.info", 'STARTER.css', "$machine_name.css");
  // Rename the .info reference to the custom js file.
  drush_op('neato_file_str_replace', "$subtheme_path/$machine_name.info", 'scripts.js', "$machine_name.js");
  // Rename functions in template.php.
  drush_op('neato_file_str_replace', "$subtheme_path/template.php", 'STARTER', "$machine_name");
  // Replace STARTER in package.json.
  drush_op('neato_file_str_replace', "$subtheme_path/package.json", 'STARTER', "$machine_name");
  // Replace STARTER in gulpfile.js.
  drush_op('neato_file_str_replace', "$subtheme_path/gulpfile.js", 'STARTER', "$machine_name");
  // Replace THEMENAME in STARTER.scss.
  drush_op('neato_file_str_replace', "$subtheme_path/scss/$machine_name.scss", 'THEMENAME', "$machine_name");
  // Fix @import paths if Neato is located inside themes/contrib.
  if ($themes_custom_path) {
    drush_op('neato_file_str_replace', "$subtheme_path/scss/$machine_name.scss", '../../neato', "../../../contrib/neato");
  }

  // Notify user of the newly created theme.
  drush_print(dt("\n!name sub-theme was created in !path. \n",
    [
      '!name' => $name,
      '!path' => $subtheme_path,
    ]
  ));

  drush_pm_enable_validate($machine_name);
  drush_pm_enable($machine_name);
}

/**
 * Internal helper: Replace strings in a file.
 */
function neato_file_str_replace($file_path, $find, $replace) {
  $file_contents = file_get_contents($file_path);
  $file_contents = str_replace($find, $replace, $file_contents);
  file_put_contents($file_path, $file_contents);
}

/**
 * Internal Helper: Remove first line of file.
 */
function neato_fix_bitters_neat_import($file_path, $themes_custom_path) {
  $file    = file($file_path);
  if ($themes_custom_path) {
    $file[0] = '@import "../../../../contrib/neato/bower_components/neat/app/assets/stylesheets/neat-helpers";';
  } else {
    $file[0] = '@import "../../../neato/bower_components/neat/app/assets/stylesheets/neat-helpers";';
  }
  file_put_contents($file_path, $file);
}

/**
 * Implements hook_drush_help().
 */
function neato_drush_help($section) {
  switch ($section) {
    case 'drush:neato-sub-theme':
      return dt("Create a sub-theme based on Neato.");
  }
}
